import { ZodError, AnyZodObject } from 'zod'
import { Request, Response, NextFunction } from 'express'
import { failed } from './responseHandler'
import { queryParser } from '@/utils/queryParser'
import { QueryCommonOptions } from '@/utils/commonSchema'

export const bodyValidator = (schema: AnyZodObject) => {
    return (req: Request, res: Response, next: NextFunction) => {
        try {
            const body = req.body
            const result = schema.parse(body)

            next()
        } catch (error) {
            if (error instanceof ZodError) {
                failed(res, 400, 'Error de validación', error.issues.map(issue => ({ message: issue.message })))
            }
            next(failed)
        }
    }
}

export const paramsValidator = (schema: AnyZodObject) => {
    return (req: Request, res: Response, next: NextFunction) => {
        try {
            const params = req.params

            const data = {
                ...params,
                id: params.id ? Number(params.id) : null
            }

            schema.parse(data)
            next()
        } catch (error) {
            if (error instanceof ZodError) {
                failed(res, 400, 'Error Zod', error.issues.map(issue => ({ message: issue.message })))
            }
            next(error)
        }
    }
}

export const queryValidator = (schema: AnyZodObject) => {
    return (req: Request, res: Response, next: NextFunction) => {
        try {
            const query: QueryCommonOptions = queryParser(req.query)

            if(query.skip != undefined && !isNaN(query.skip)){
                query.skip = Number(query.skip)
            }

            if (query.take != undefined && !isNaN(query.take)) {
                query.take = Number(query.take)
            }

            schema.parse(query)  

            next()
        } catch (error) {
            if (error instanceof ZodError) {
                failed(res, 400, 'Error Zod', error.issues.map(issue => ({ message: issue.message })))
            }
            next(failed)
        }
    }
}