import { Router, Request, Response, NextFunction } from 'express'
import { getAccounts, getAccount, openAccount, deleteAccount, closeAccount } from './accountController'
import { createAccountSchema, updateAccountSchema } from './accountSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const accounts = await getAccounts(req.query)
            success(res, accounts)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const accountId = Number(req.params.id)
            const account = await getAccount(accountId)
            success(res, { account }, 'Caja encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createAccountSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const account = await openAccount(req.body)
            success(res, { account }, 'Caja aperturada con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateAccountSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const accountId = Number(req.params.id)
            const account = await closeAccount(accountId, req.body)
            success(res, { account }, 'Caja cerrada con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const accountId = Number(req.params.id)
            await deleteAccount(accountId)
            success(res, {}, 'Caja eliminado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router