import { Router, Request, Response, NextFunction } from 'express'
import { getBrands, getBrand, createBrand, deleteBrand, updateCategoty } from './brandController'
import { createBrandSchema, updateBrandSchema } from './brandSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { queryParser } from '@/utils/queryParser'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const query = queryParser(req.query)
            const brands = await getBrands(query)
            success(res, brands)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const brandId = Number(req.params.id)
            const brand = await getBrand(brandId)
            success(res, { brand }, 'Marca encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createBrandSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const brand = await createBrand(req.body)
            success(res, { brand }, 'Marca creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateBrandSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const brandId = Number(req.params.id)
            const brand = await updateCategoty(brandId, req.body)
            success(res, { brand }, 'Marca actualizado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const brandId = Number(req.params.id)
            await deleteBrand(brandId)
            success(res, {}, 'Marca eliminado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router