import boom from '@hapi/boom'
import { Cashier } from '@/db/entities/Cashier'
import { CreateCashierOptions, CloseCashierOptions } from './cashierSchema'
import { QueryOptions } from '@/@types/common'
import { getUser } from '../users/userController'
import { CashierStatus } from '@/db/Common'
import { dataSource } from '@/bootstrap/dbBootstrap'

type CashiersQueryOptions = QueryOptions & { status?: unknown }

export const getCashiers = async (query: CashiersQueryOptions) => {
    const { skip, take, search, sortKey, sortOrder, status } = query

    const queryBuilder = dataSource.createQueryBuilder(Cashier, "cashier")
        .innerJoin("cashier.user", "user")
        .select([
            "cashier.id",
            "cashier.createdAt",
            "cashier.status",
            "cashier.initialBalance",
            "user.id",
            "user.username",
            "cashier.updatedAt"
        ])

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('cashier.updatedAt', 'DESC')
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere("user.username ILIKE :search", { search: `%${search}%` })
    }

    if (Array.isArray(status) && status.length > 0) {
        queryBuilder.andWhere("cashier.status IN (:...status)", { status });
    }

    const [cashiers, total] = await queryBuilder.getManyAndCount()

    return { cashiers, total }
}

export const getCashier = async (id: number) => {
    const cashier = await Cashier.findOneBy({ id })

    if (!cashier) {
        throw boom.notFound('No existe la caja')
    }

    return cashier
}

export const getCashierOpeningByUser = async (userId: number) => {
    const query = `
            SELECT
    c.id,
    c."createdAt",
    c.status,
    c."initialBalance",
    u.username,
    u.fullname,
    COALESCE(e."totalExpense", 0) AS "totalExpense",
    COALESCE(s."totalSale", 0) AS "totalSale"
FROM cashiers c
INNER JOIN users u ON u.id = c."userId"
LEFT JOIN (
    SELECT "cashierId", SUM(amount) AS "totalExpense"
    FROM expenses
    GROUP BY "cashierId"
) e ON e."cashierId" = c.id
LEFT JOIN (
    SELECT "cashierId", SUM("saleAmount" + igv) AS "totalSale"
    FROM sales
    GROUP BY "cashierId"
) s ON s."cashierId" = c.id
WHERE u.id = $1 AND c.status = $2
        `
    const cashiers = await dataSource.query(query, [userId, CashierStatus.OPENING])

    if (Array.isArray(cashiers) && cashiers.length > 0) {
        return cashiers[0]
    } else {
        throw boom.notAcceptable('El usuario no tiene una caja aperturada')
    }
}

export const getCashierDetail = async (cashierId: number) => {
    const query = `
    SELECT
    c.id,
    c."createdAt",
    c.status,
    c."initialBalance",
    u.username,
    u.fullname,
    COALESCE(e."totalExpense", 0) AS "totalExpense",
    COALESCE(s."totalSale", 0) AS "totalSale"
FROM cashiers c
INNER JOIN users u ON u.id = c."userId"
LEFT JOIN (
    SELECT "cashierId", SUM(amount) AS "totalExpense"
    FROM expenses
    GROUP BY "cashierId"
) e ON e."cashierId" = c.id
LEFT JOIN (
    SELECT "cashierId", SUM("saleAmount" + igv) AS "totalSale"
    FROM sales
    GROUP BY "cashierId"
) s ON s."cashierId" = c.id
WHERE c.id = $1
    `
    const cashiers = await dataSource.query(query, [cashierId])

    if (Array.isArray(cashiers) && cashiers.length > 0) {
        return cashiers[0]
    } else {
        throw boom.notAcceptable('La caja no existe')
    }
}

export const validateCashierOpeningByUser = async (userId: number) => {
    const cashier = await Cashier.findOne({
        relations: {
            user: true
        },
        where: {
            status: CashierStatus.OPENING,
            user: {
                id: userId
            }
        }
    })

    return cashier
}

export const openCashier = async (data: CreateCashierOptions) => {
    const { initialBalance, userId } = data
    const user = await getUser(userId)
    const cashier = new Cashier()

    const userCashier = await validateCashierOpeningByUser(user.id)

    if (userCashier) {
        throw boom.notAcceptable('El usuario tiene una caja aperturada')
    }

    cashier.initialBalance = initialBalance
    cashier.user = user

    await cashier.save()

    if (!cashier) {
        throw boom.badData('No se puede aperturar la caja')
    }

    return cashier
}

export const closeCashier = async (id: number) => {
    // const { closingDatetime, unbalance, observation } = changes
    const cashier = await getCashier(id)
    // cashier.closingDatetime = closingDatetime.toString()
    // cashier.unbanlance = unbalance
    // cashier.observation = observation
    cashier.status = CashierStatus.CLOSING

    await cashier.save()

    if (!cashier) {
        throw boom.badData('No se puede cerrar la caja')
    }

    return cashier
}

export const deleteCashier = async (id: number) => {
    const cashier = await getCashier(id)
    await cashier.remove()

    return cashier
}