import { Router, Request, Response, NextFunction } from 'express'
import { getCashiers, getCashier, openCashier, deleteCashier, closeCashier, getCashierOpeningByUser, getCashierDetail } from './cashierController'
import { openCashierSchema, closeCashierSchema } from './cashierSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { CustomAuthRequest } from '@/@types/common'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const cashiers = await getCashiers(req.query)
            success(res, cashiers)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/detail/:cashierId',
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const cashierId = Number(req.params.cashierId)

            const cashier = await getCashierDetail(cashierId)
            success(res, { cashier }, 'Caja encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.get('/owner',
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const userId = req.user.sub

            const cashier = await getCashierOpeningByUser(userId)
            success(res, { cashier }, 'Caja encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const cashierId = Number(req.params.id)
            const cashier = await getCashierOpeningByUser(cashierId)
            success(res, { cashier }, 'Caja encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(openCashierSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const cashier = await openCashier(req.body)
            success(res, { cashier }, 'Caja aperturada con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id/closing',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const cashierId = Number(req.params.id)
            const cashier = await closeCashier(cashierId)
            success(res, { cashier }, 'Caja cerrada con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const cashierId = Number(req.params.id)
            await deleteCashier(cashierId)
            success(res, {}, 'Caja eliminado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router