import { Router, Request, Response, NextFunction } from 'express'
import { getCategories, getCategory, createCategory, deleteCategory, updateCategoty } from './categoryController'
import { createCategorySchema, updateCategorySchema } from './categorySchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { queryParser } from '@/utils/queryParser'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const query = queryParser(req.query)
            const categories = await getCategories(query)
            success(res, categories)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const categoryId = Number(req.params.id)
            const category = await getCategory(categoryId)
            success(res, { category }, 'Categoría encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createCategorySchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const category = await createCategory(req.body)
            success(res, { category }, 'Categoría creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateCategorySchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const categoryId = Number(req.params.id)
            const category = await updateCategoty(categoryId, req.body)
            success(res, { category }, 'Categoría actualizado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const categoryId = Number(req.params.id)
            await deleteCategory(categoryId)
            success(res, {}, 'Categoría eliminada con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router