import { FindManyOptions, ILike } from 'typeorm'
import boom from '@hapi/boom'
import { Customer } from '@/db/entities/Customer'
import { CreateCustomerOptions, UpdateCustomerOptions } from './customerSchema'
import { QueryOptions } from '@/@types/common'
import { dataSource } from '@/bootstrap/dbBootstrap'

type CustomerQueryOptions = QueryOptions & { documentType?: unknown }

export const getCustomers = async (query: CustomerQueryOptions) => {
    const { skip, take, search, sortKey, sortOrder, documentType } = query

    const queryBuilder = dataSource.createQueryBuilder(Customer, "customer")
        .select([
            "customer.id",
            "customer.name",
            "customer.documentType",
            "customer.documentNumber",
            "customer.phoneNumber",
            "customer.address",
            "customer.email",
        ])

    if (Array.isArray(documentType) && documentType.length > 0) {
        queryBuilder.andWhere("customer.documentType IN (:...documentType)", { documentType })
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere(
            `(customer.name ILIKE :search OR
            customer.documentNumber ILIKE :search OR
            customer.phoneNumber ILIKE :search OR
            customer.email ILIKE :search OR
            customer.address ILIKE :search)`,
            { search: `%${search}%` }
        )
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('customer.updatedAt', 'DESC')
    }

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    const [customers, total] = await queryBuilder.getManyAndCount()

    return { customers, total }
}

export const getCustomer = async (id: number) => {
    const customer = await Customer.findOneBy({ id })

    if (!customer) {
        throw boom.notFound('No se ha encontrado el cliente')
    }

    return customer
}

export const createCustomer = async (data: CreateCustomerOptions) => {
    const { name, documentNumber, documentType, address, phoneNumber, email } = data
    const customer = new Customer()
    customer.name = name
    customer.documentNumber = documentNumber
    customer.documentType = documentType
    customer.address = address
    customer.phoneNumber = phoneNumber
    customer.email = email

    await customer.save()

    if (!customer) {
        throw boom.badData('No se puede registrar el cliente')
    }

    return customer
}

export const updateCustomer = async (id: number, changes: UpdateCustomerOptions) => {
    const { name, documentNumber, documentType, address, phoneNumber, email } = changes
    const customer = await getCustomer(id)
    customer.name = name
    customer.documentNumber = documentNumber
    customer.documentType = documentType
    customer.address = address
    customer.phoneNumber = phoneNumber
    customer.email = email

    await customer.save()

    if (!customer) {
        throw boom.badData('No se puede actualizar el cliente')
    }

    return customer
}

export const deleteCustomer = async (id: number) => {
    const customer = await getCustomer(id)
    await customer.remove()

    return customer
}