import { Router, Request, Response, NextFunction } from 'express'
import { getCustomers, getCustomer, createCustomer, deleteCustomer, updateCustomer } from './customerController'
import { createCustomerSchema, updateCustomerSchema } from './customerSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const customers = await getCustomers(req.query)
            success(res, customers)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const customerId = Number(req.params.id)
            const customer = await getCustomer(customerId)
            success(res, { customer }, 'Cliente encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createCustomerSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const customer = await createCustomer(req.body)
            success(res, { customer }, 'Cliente creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateCustomerSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const customerId = Number(req.params.id)
            const customer = await updateCustomer(customerId, req.body)
            success(res, { customer }, 'Cliente actualizado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const customerId = Number(req.params.id)
            await deleteCustomer(customerId)
            success(res, {}, 'Cliente eliminado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router