import { FindManyOptions, ILike } from 'typeorm'
import boom from '@hapi/boom'
import { Expense } from '@/db/entities/Expense'
import { CreateExpense, UpdateExpense } from './expenseSchema'
import { QueryOptions } from '@/@types/common'
import { getConcept } from '../concepts/conceptController'
import { getCashierOpeningByUser } from '../cashiers/cashierController'
import { dataSource } from '@/bootstrap/dbBootstrap'
import { CashierStatus } from '@/db/Common'

export const getExpenses = async (query: QueryOptions) => {
    const { skip, take, search, sortKey, sortOrder } = query

    const queryBuilder = dataSource.createQueryBuilder(Expense, "expense")
        .innerJoin("expense.concept", "concept")
        .select([
            "cashier.id",
            "cashier.description",
            "cashier.amount",
            "concept.id",
            "concept.name"
        ])

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('concept.updatedAt', 'DESC')
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere("expense.description ILIKE :search", { search: `%${search}%` })
    }

    if (Array.isArray(status) && status.length > 0) {
        queryBuilder.andWhere("concept.id IN (:...status)", { status });
    }

    const [expenses, total] = await queryBuilder.getManyAndCount()

    return { expenses, total }
}

export const getExpensesOwner = async (query: QueryOptions, userId: number) => {
    const { skip, take, search, sortKey, sortOrder } = query

    const queryBuilder = dataSource.createQueryBuilder(Expense, "expense")
        .innerJoin("expense.concept", "concept")
        .innerJoin("expense.cashier", "cashier")
        .select([
            "expense.id",
            "expense.description",
            "expense.amount",
            "concept.id",
            "concept.name",
            "expense.updatedAt"
        ])
        .where("cashier.userId = :userId", { userId })
        .andWhere("cashier.status = :cashierStatus", { cashierStatus: CashierStatus.OPENING })

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } 
    else {
        queryBuilder.orderBy('expense.updatedAt', 'DESC')
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere("expense.description ILIKE :search", { search: `%${search}%` })
    }

    const [expenses, total] = await queryBuilder.getManyAndCount()

    return { expenses, total }
}

export const getExpensesByCashier = async (query: QueryOptions, cashierId: number) => {
    const { skip, take, search, sortKey, sortOrder } = query

    const queryBuilder = dataSource.createQueryBuilder(Expense, "expense")
        .innerJoin("expense.concept", "concept")
        .innerJoin("expense.cashier", "cashier")
        .select([
            "expense.id",
            "expense.description",
            "expense.amount",
            "concept.id",
            "concept.name",
            "expense.updatedAt"
        ])
        .where("cashier.id = :cashierId", { cashierId })

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    }
    else {
        queryBuilder.orderBy('expense.updatedAt', 'DESC')
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere("expense.description ILIKE :search", { search: `%${search}%` })
    }

    const [expenses, total] = await queryBuilder.getManyAndCount()

    return { expenses, total }
}

export const getExpense = async (id: number) => {
    const expense = await Expense.findOneBy({ id })

    if (!expense) {
        throw boom.notFound('No se ha encontrado el gasto')
    }

    return expense
}

export const createExpense = async (data: CreateExpense, userId: number) => {
    const { description, conceptId, amount } = data
    const concept = await getConcept(conceptId)
    const cashier = await getCashierOpeningByUser(userId)
    const expense = new Expense()

    expense.description = description
    expense.amount = amount
    expense.concept = concept
    expense.cashier = cashier

    await expense.save()

    if (!expense) {
        throw boom.badData('No se puede crear el Gasto')
    }

    return expense
}

export const updateCategoty = async (id: number, changes: UpdateExpense) => {
    const { description, amount, conceptId } = changes
    const concept = await getConcept(conceptId)
    const expense = await getExpense(id)

    expense.description = description
    expense.amount = amount
    expense.concept = concept

    await expense.save()

    if (!expense) {
        throw boom.badData('No se puede actualizar el Gasto')
    }

    return expense
}

export const deleteExpense = async (id: number) => {
    const expense = await getExpense(id)
    await expense.remove()

    return expense
}