import { Router, Request, Response, NextFunction } from 'express'
import { getExpenses, getExpense, createExpense, deleteExpense, updateCategoty, getExpensesOwner, getExpensesByCashier } from './expenseController'
import { createExpenseSchema, updateExpenseSchema } from './expenseSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { CustomAuthRequest } from '@/@types/common'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const expenses = await getExpenses(req.query)
            success(res, expenses)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/owner',
    queryValidator(queryCommonSchema),
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const userId = req.user.sub
            const expenses = await getExpensesOwner(req.query, userId)
            success(res, expenses)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/detail/:cashierId',
    queryValidator(queryCommonSchema),
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const cashierId = Number(req.params.cashierId)
            const expenses = await getExpensesByCashier(req.query, cashierId)
            success(res, expenses)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const expenseId = Number(req.params.id)
            const expense = await getExpense(expenseId)
            success(res, { expense }, 'Gasto encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createExpenseSchema),
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const userId = req.user.sub
            const expense = await createExpense(req.body, userId)
            success(res, { expense }, 'Gasto creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateExpenseSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const expenseId = Number(req.params.id)
            const expense = await updateCategoty(expenseId, req.body)
            success(res, { expense }, 'Gasto actualizado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const expenseId = Number(req.params.id)
            await deleteExpense(expenseId)
            success(res, {}, 'Gasto eliminado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router