import { FindManyOptions, ILike, In } from 'typeorm'
import { dataSource } from '@/bootstrap/dbBootstrap'
import boom from '@hapi/boom'
import { Product } from '@/db/entities/Product'
import { CreateProductOptions, UpdateProductOptions } from './productSchema'
import { QueryCommonOptions } from '@/utils/commonSchema'
import { getUnit } from '../units/unitController'
import { getBrand } from '../brands/brandController'
import { getSubcategory } from '../subcategories/subcategoryController'
import { QueryOptions } from '@/@types/common'
import { SearchProductsData } from '@/@types/products'


type ProductQueryOptions = QueryOptions & { categorySlug?: string, subcategorySlug?: string }

export const getProducts = async (query: ProductQueryOptions) => {
    const { skip, take, search, sortKey, sortOrder, categorySlug, subcategorySlug } = query

    const queryBuilder = dataSource.createQueryBuilder(Product, "product")
        .innerJoin("product.subcategory", "subcategory")
        .innerJoin("product.brand", "brand")
        .innerJoin("product.unit", "unit")
        .innerJoin("subcategory.category", "category")
        .select([
            "product.id",
            "product.sku",
            "product.name",
            "product.imageUrl",
            "product.cost",
            "product.price",
            "product.stock",
            "product.stockMin",
            "product.updatedAt",
            "subcategory.name",
            "category.name",
            "brand.name",
            "unit.abbreviation"
        ])

    if (categorySlug != undefined && categorySlug != '') {
        queryBuilder.where("category.slug = :categorySlug", { categorySlug })
        if (subcategorySlug != undefined && subcategorySlug != '') {
            queryBuilder.andWhere("subcategory.slug = :subcategorySlug", { subcategorySlug })
        }
    }

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('product.updatedAt', 'DESC')
    }

    // if (Array.isArray(brands) && brands.length > 0) {
    //     queryBuilder.andWhere("product.brandId IN (:...brands)", { brands });
    // }

    if (search) {
        queryBuilder.andWhere("product.name ILIKE :search", { search: `%${search}%` });
    }

    const [products, total] = await queryBuilder.getManyAndCount();

    return { products, total }
}

// export const getProductsBySubcategory = async (categorySlug: string, subcategorySlug: string, query: QueryOptions, data: SearchProductsData) => {
//     const { skip, take, search, sortKey, sortOrder } = query
//     const { brands } = data

//     const queryOptions = {
//         order: {
//             id: 'DESC'
//         },
//         relations: {
//             subcategory: true
//         },
//         where: {
//             subcategory: {
//                 slug: subcategorySlug
//             }
//         }
//     } as FindManyOptions

//     if(skip && take) {
//         queryOptions.skip = skip as number
//         queryOptions.take = take as number
//     }

//     if (sortKey && sortOrder) {
//         queryOptions.order = {
//             [sortKey]: sortOrder
//         }
//     }

//     if (Array.isArray(brands) && brands.length > 0) {
//         queryOptions.where = {
//             ...queryOptions.where,
//             brand: {
//                 id: In(brands)
//             }
//         }
//     }

//     if (search) {
//         queryOptions.where = {
//             ...queryOptions.where,
//             name: ILike(`%${search}%`)
//         }
//     }

//     const products = await Product.findAndCount(queryOptions)

//     return { products: products[0], total: products[1] }
// }

export const getProductsByCategory = async (categorySlug: string, subcategorySlug: string, query: QueryOptions, data: SearchProductsData) => {
    const { skip, take, search, sortKey, sortOrder } = query
    const { brands } = data

    const queryBuilder = dataSource.createQueryBuilder(Product, "product")
        .innerJoin("product.subcategory", "subcategory")
        .innerJoin("product.brand", "brand")
        .innerJoin("product.unit", "unit")
        .innerJoin("subcategory.category", "category")
        .select([
            "product.id",
            "product.sku",
            "product.name",
            "product.cost",
            "product.price",
            "product.stock",
            "product.stockMin",
            "subcategory.name",
            "category.name",
            "brand.name",
            "unit.abbreviation"
        ])
        .where("subcategory.slug = :subcategorySlug", { subcategorySlug })
        .andWhere("category.slug = :categorySlug", { categorySlug })

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey && sortOrder) {
        queryBuilder.orderBy(`product.${sortKey}`, sortOrder)
    }

    if (Array.isArray(brands) && brands.length > 0) {
        queryBuilder.andWhere("product.brandId IN (:...brands)", { brands });
    }

    if (search) {
        queryBuilder.andWhere("product.name ILIKE :search", { search: `%${search}%` });
    }

    const [products, total] = await queryBuilder.getManyAndCount();

    return { products, total };
}

export const getProduct = async (id: number) => {
    const product = await Product.findOne({
        where: {
            id
        },
        relations: {
            brand: true,
            subcategory: true,
            unit: true,
        }
    })

    if (!product) {
        throw boom.notFound('Producto no existe')
    }

    return product
}

export const createProduct = async (data: CreateProductOptions) => {
    const { sku, name, cost, utility, price, stock, stockMin, bulkDiscount, unitId, subcategoryId, brandId, imageUrl } = data
    const unit = await getUnit(unitId)
    const brand = await getBrand(brandId)
    const subcategory = await getSubcategory(subcategoryId)
    const product = new Product()

    product.sku = sku
    product.name = name
    product.cost = cost
    product.utility = utility
    product.price = price
    product.stock = stock
    product.stockMin = stockMin
    product.bulkDiscount = bulkDiscount
    product.unit = unit
    product.brand = brand
    product.subcategory = subcategory
    product.imageUrl = imageUrl

    await product.save()

    if (product) {
        return product
    } else {
        throw boom.badData('No se puede registrar producto')
    }
}

export const updateCategoty = async (id: number, changes: UpdateProductOptions) => {
    const { sku, name, cost, utility, price, stock, stockMin, bulkDiscount, unitId, subcategoryId, brandId, imageUrl } = changes
    const unit = await getUnit(unitId)
    const brand = await getBrand(brandId)
    const subcategory = await getSubcategory(subcategoryId)
    const product = await getProduct(id)
    product.sku = sku
    product.name = name
    product.cost = cost
    product.utility = utility
    product.price = price
    product.stock = stock
    product.stockMin = stockMin
    product.bulkDiscount = bulkDiscount
    product.unit = unit
    product.brand = brand
    product.subcategory = subcategory
    product.imageUrl = imageUrl

    await product.save()

    if (product) {
        return product
    } else {
        throw boom.badData('No se puede actualizar producto')
    }
}

export const deleteProduct = async (id: number) => {
    const product = await getProduct(id)
    await product.remove()

    return product
}