import { FindManyOptions, ILike } from 'typeorm'
import boom from '@hapi/boom'
import { Sale } from '@/db/entities/Sale'
import { ProductSale } from '@/db/entities/ProductSale'
import { CreateSaleOptions } from './saleSchema'
import { QueryOptions } from '@/@types/common'
import { dataSource } from '@/bootstrap/dbBootstrap'
import { CashierStatus, InvoceTypes } from '@/db/Common'
import { getCashierOpeningByUser } from '../cashiers/cashierController'
import { getUpdateConfig } from '../configs/configController'

type SaleQueryOptions = QueryOptions & { invoceType?: unknown }

export const getSales = async (query: SaleQueryOptions) => {
    const { skip, take, search, sortKey, sortOrder, invoceType } = query

    const queryBuilder = dataSource.createQueryBuilder(Sale, "sale")
        .select([
            "sale.id",
            "sale.name",
            "sale.documentType",
            "sale.documentNumber",
            "sale.phoneNumber",
            "sale.address",
            "sale.email",
        ])

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('sale.updatedAt', 'DESC')
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere(
            `sale.name ILIKE :search OR
            sale.documentNumber ILIKE :search OR
            sale.phoneNumber ILIKE :search OR
            sale.email ILIKE :search OR
            sale.address ILIKE :search`,
            { search: `%${search}%` })
    }

    if (Array.isArray(invoceType) && invoceType.length > 0) {
        queryBuilder.andWhere("sale.documentType IN (:...documentType)", { invoceType })
    }

    const [sales, total] = await queryBuilder.getManyAndCount()

    return { sales, total }
}

export const getSalesOwner = async (query: SaleQueryOptions, userId: number) => {
    const { skip, take, search, sortKey, sortOrder, invoceType } = query

    const queryBuilder = dataSource.createQueryBuilder(Sale, "sale")
        .innerJoin("sale.customer", "customer")
        .innerJoin("sale.cashier", "cashier")
        .select([
            "sale.id",
            "sale.invoceNumber",
            "sale.invoceType",
            "sale.saleAmount",
            "sale.igv",
            "sale.updatedAt",
            "sale.createdAt",
            "customer.name",
            "customer.documentNumber",
            "customer.documentType",
        ])
        .where("cashier.userId = :userId", { userId })
        .andWhere("cashier.status = :cashierStatus", { cashierStatus: CashierStatus.OPENING })

    if (Array.isArray(invoceType) && invoceType.length > 0) {
        queryBuilder.andWhere("sale.invoceType IN (:...invoceType)", { invoceType })
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere(`
            (sale.invoceNumber ILIKE :search OR
            customer.name ILIKE :search OR
            customer.documentNumber ILIKE :search)`,
            { search: `%${search}%` }
        )
    }

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('sale.updatedAt', 'DESC')
    }

    const [sales, total] = await queryBuilder.getManyAndCount()

    return { sales, total }
}

export const getSalesByCashier = async (query: SaleQueryOptions, cashierId: number) => {
    const { skip, take, search, sortKey, sortOrder, invoceType } = query

    const queryBuilder = dataSource.createQueryBuilder(Sale, "sale")
        .innerJoin("sale.customer", "customer")
        .innerJoin("sale.cashier", "cashier")
        .select([
            "sale.id",
            "sale.invoceNumber",
            "sale.invoceType",
            "sale.saleAmount",
            "sale.igv",
            "sale.updatedAt",
            "sale.createdAt",
            "customer.name",
            "customer.documentNumber",
            "customer.documentType",
        ])
        .where("cashier.id = :cashierId", { cashierId })

    if (Array.isArray(invoceType) && invoceType.length > 0) {
        queryBuilder.andWhere("sale.invoceType IN (:...invoceType)", { invoceType })
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere(`
            (sale.invoceNumber ILIKE :search OR
            customer.name ILIKE :search OR
            customer.documentNumber ILIKE :search)`,
            { search: `%${search}%` }
        )
    }

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    }
    else {
        queryBuilder.orderBy('sale.updatedAt', 'DESC')
    }

    const [sales, total] = await queryBuilder.getManyAndCount()

    return { sales, total }
}

export const getSale = async (id: number) => {
    const sale = await Sale.findOne({
        where: {
            id
        },
        relations: {
            items: {
                product: {
                    unit: true
                }
            },
            customer: true
        }
    })

    if (!sale) {
        throw boom.notFound('No se ha encontrado el cliente')
    }

    return sale
}

export const createSale = async (data: CreateSaleOptions, userId: number) => {
    const { saleAmount, igv, invoceType, customerId, items } = data
    const cashier = await getCashierOpeningByUser(userId)
    const invoceNumber = await getUpdateConfig(invoceType)

    const result = await dataSource
        .createQueryBuilder()
        .insert()
        .into(Sale)
        .values([
            {
                invoceNumber: invoceNumber,
                invoceType,
                igv,
                saleAmount,
                cashierId: cashier.id,
                customerId
            }
        ])
        .execute()

    const saleId = result.identifiers[0].id

    if (saleId) {
        const products = items.map(item => ({
            ...item,
            saleId
        }))

        await dataSource
            .createQueryBuilder()
            .insert()
            .into(ProductSale)
            .values(products)
            .execute();
    } else {
        throw boom.badData('No se puede guardar la venta')
    }
}