import { Router, Request, Response, NextFunction } from 'express'
import { getSales, getSale, createSale, getSalesOwner, getSalesByCashier } from './saleController'
import { createSaleSchema } from './saleSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { CustomAuthRequest } from '@/@types/common'
import fs from 'fs'
import path from 'path'
import pdfMake from 'pdfmake'
import { TDocumentDefinitions } from 'pdfmake/interfaces'
import dayjs from 'dayjs'
import 'dayjs/locale/es'

dayjs.locale('es')

const fonts = {
    Roboto: {
        normal: 'src/fonts/Roboto-Regular.ttf',
        bold: 'src/fonts/Roboto-Medium.ttf',
        italics: 'src/fonts/Roboto-Italic.ttf',
        bolditalics: 'src/fonts/Roboto-MediumItalic.ttf'
    }
}
const printer = new pdfMake(fonts)
const router = Router()
const imagePath = 'src/public/images/inspirationic-logo.png'
const imageBase64 = fs.readFileSync(imagePath, { encoding: 'base64' })

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const sales = await getSales(req.query)
            success(res, sales)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/owner',
    queryValidator(queryCommonSchema),
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const userId = req.user.sub
            const expenses = await getSalesOwner(req.query, userId)
            success(res, expenses)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/detail/:cashierId',
    queryValidator(queryCommonSchema),
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const cashierId = Number(req.params.cashierId)
            const sales = await getSalesByCashier(req.query, cashierId)
            success(res, sales)
        } catch (error) {
            next(error)
        }
    }
)

// router.get('/:id',
//     paramsValidator(paramsCommonSchema),
//     async (req: Request, res: Response, next: NextFunction) => {
//         try {
//             const saleId = Number(req.params.id)
//             const sale = await getSale(saleId)

//             // success(res, { sale }, 'Cliente encontrado con éxito')

//             const doc = new PDFDocument({
//                 size: [mmToPoints(80), mmToPoints(150)],
//                 margin: 0.5
//             })

//             res.setHeader('Content-disposition', 'attachment; filename=sale.pdf')
//             res.setHeader('Content-type', 'application/pdf')

//             doc.fontSize(12).text(`${sale.invoceType.toUpperCase()}`, { align: 'center' })
//             doc.fontSize(12).text(`#${sale.invoceNumber}`, { align: 'center' })
//             doc.text(`Fecha: ${sale.createdAt}`)
//             sale.items.forEach((item: any) => {
//                 doc.text(`${item.product.name} - ${item.unitPrice}`)
//             });
//             doc.text(`Total - ${sale.saleAmount}`)

//             doc.end()
//             doc.pipe(res)
//         } catch (error) {
//             next(error)
//         }
//     }
// )

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const saleId = Number(req.params.id)
            const sale = await getSale(saleId)
            // success(res, sale)
            const formattedDate = dayjs(sale.createdAt).format('D [de] MMMM [de] YYYY, h:mm A')
            const totalSale = parseFloat(sale.igv.toString()) + parseFloat(sale.saleAmount.toString())

            const docDefinition: TDocumentDefinitions = {
                pageSize: {
                    width: 200, // 120 mm de ancho
                    height: 'auto' // Altura dinámica
                },
                pageMargins: [5, 10, 5, 10], // Margen de 0.5 mm en todos los lados
                content: [
                    { image: `data:image/png;base64,${imageBase64}`, width: 50, alignment: 'center', margin: [0, 0, 0, 2] },
                    { text: 'INSPIRATIONIC CORP S.A.C.', style: 'header', alignment: 'center' },
                    { text: 'RUC: 20345678937', style: 'subheader', alignment: 'center' },
                    { text: 'Tel: 940600315', style: 'subheader', alignment: 'center' },
                    { text: 'https://inspirationic.com', style: 'subheader', alignment: 'center' },
                    {
                        canvas: [
                            {
                                type: 'line',
                                x1: 0, y1: 5,
                                x2: 190, y2: 5,  // Largo de la línea (515 es el ancho completo de la página A4)
                                lineWidth: 0.5,   // Grosor de la línea
                                dash: { length: 3, space: 2 }
                            }
                        ]
                    },
                    { text: `${sale.invoceType.toUpperCase()}`, style: 'header', alignment: 'center', margin: [0, 10, 0, 0] },
                    { text: `#${sale.invoceNumber}`, style: 'subheader', alignment: 'center', margin: [0, 0, 0, 10] },
                    { text: `${sale.customer.name}`, style: 'normal' },
                    { text: `${sale.customer.documentType}: ${sale.customer.documentNumber}`, style: 'normal' },
                    { text: `Fecha de emisión: ${formattedDate}`, style: 'normal' },
                    {
                        canvas: [
                            {
                                type: 'line',
                                x1: 0, y1: 5,
                                x2: 190, y2: 5,  // Largo de la línea (515 es el ancho completo de la página A4)
                                lineWidth: 0.5,   // Grosor de la línea
                                dash: { length: 3, space: 2 },
                            },
                        ]
                    },
                    { text: 'CANT. / DESCRIPTION / P.UNIT. / IMPORTE', style: 'normal', margin: [0, 10, 0, 0] },
                    ...sale.items.map((item: any) => {
                        return { text: `${item.quantity}${item.product.unit.abbreviation}  ${item.product.name}  s/.${item.unitPrice} s/.${(item.unitPrice * item.quantity).toFixed(2)}`, style: 'normal' }
                    }),
                    { text: `SUBTOTAL: s/.${sale.saleAmount}`, style: 'normal', alignment: 'right', margin: [0, 10, 0, 0] },
                    { text: `IGV: s/.${sale.igv}`, style: 'normal', alignment: 'right' },
                    { text: `TOTAL: s/.${totalSale.toFixed(2)}`, style: 'normal', alignment: 'right', margin: [0, 0, 0, 10] },
                    {
                        canvas: [
                            {
                                type: 'line',
                                x1: 0, y1: 5,
                                x2: 190, y2: 5,  // Largo de la línea (515 es el ancho completo de la página A4)
                                lineWidth: 0.5,   // Grosor de la línea
                                dash: { length: 3, space: 2 }
                            }
                        ]
                    },
                    { text: '¡Gracias por su compra!', style: 'normal', alignment: 'center', margin: [0, 10, 0, 0] },
                ],
                styles: {
                    header: {
                        fontSize: 10,
                        bold: true
                    },
                    subheader: {
                        fontSize: 9,
                    },
                    normal: {
                        fontSize: 8
                    },
                    total: {
                        fontSize: 9,
                        bold: true,
                        margin: [0, 10, 0, 0]
                    }
                }
            }

            // Crear el PDF usando PDFMake
            const pdfDoc = printer.createPdfKitDocument(docDefinition)

            res.setHeader('Content-disposition', 'attachment; filename=sale.pdf')
            res.setHeader('Content-type', 'application/pdf')

            // Enviar el PDF como respuesta
            pdfDoc.end()
            pdfDoc.pipe(res)
        } catch (error) {
            next(error)
        }
    }
);

router.post('/',
    bodyValidator(createSaleSchema),
    async (req: CustomAuthRequest, res: Response, next: NextFunction) => {
        try {
            const userId = req.user.sub
            await createSale(req.body, userId)
            success(res, {}, 'Venta creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

export default router