import { FindManyOptions, ILike } from 'typeorm'
import boom from '@hapi/boom'
import { Subcategory } from '@/db/entities/Subcategory'
import { CreateSubcategoryOptions, UpdateSubcategoryOptions } from './subcategorySchema'
import { getCategory } from '../categories/categoryController'
import { QueryOptions } from '@/@types/common'

export const getSubcategories = async (query: QueryOptions) => {
    const { skip, take, search, sortKey, sortOrder } = query
    const queryOptions = {
        order: {
            id: 'DESC'
        },
        skip: skip,
        take: take
    } as FindManyOptions

    if (sortKey && sortOrder) {
        queryOptions.order = {
            [sortKey]: sortOrder
        }
    }

    if (search) {
        queryOptions.where = {
            name: ILike(`%${search}%`)
        }
    }

    const subcategories = await Subcategory.findAndCount(queryOptions)

    return { subcategories: subcategories[0], total: subcategories[1] }
}

export const getSubcategory = async (id: number) => {
    const subcategory = await Subcategory.findOneBy({ id })

    if (!subcategory) {
        throw boom.notFound('Subcategoría no existe')
    }

    return subcategory
}

export const createSubcategory = async (data: CreateSubcategoryOptions) => {
    const { name, categoryId } = data
    const category = await getCategory(categoryId)
    const subcategory = new Subcategory()

    subcategory.name = name
    subcategory.category = category

    await subcategory.save()

    if (!subcategory) {
        throw boom.badData('No se puede crear subcategoría')
    }

    const newCategory = await getCategory(categoryId)
    return { category: newCategory, subcategory }
}

export const updateSubcategory = async (id: number, changes: UpdateSubcategoryOptions) => {
    const { name, categoryId } = changes
    const category = await getCategory(categoryId)
    const subcategory = await getSubcategory(id)

    subcategory.name = name
    subcategory.category = category

    await subcategory.save()

    if (!subcategory) {
        throw boom.badData('No se puede actualizar subcategoría')
    }

    const newCategory = await getCategory(categoryId)
    return { category: newCategory, subcategory }
}

export const deleteSubcategory = async (id: number) => {
    const subcategory = await getSubcategory(id)
    await subcategory.remove()
    return id
}