import { Router, Request, Response, NextFunction } from 'express'
import { getSubcategories, getSubcategory, createSubcategory, deleteSubcategory, updateSubcategory } from './subcategoryController'
import { createSubcategorySchema, updateSubcategorySchema } from './subcategorySchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { queryParser } from '@/utils/queryParser'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const query = queryParser(req.query)
            const subcategories = await getSubcategories(query)
            success(res, subcategories)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const subcategoryId = Number(req.params.id)
            const subcategory = await getSubcategory(subcategoryId)
            success(res, { subcategory }, 'Subcategoría encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createSubcategorySchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const result = await createSubcategory(req.body)
            success(res, result, 'Subcategoría creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateSubcategorySchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const subcategoryId = Number(req.params.id)
            const result = await updateSubcategory(subcategoryId, req.body)
            success(res, result, 'Subcategoría actualizado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const subcategoryId = Number(req.params.id)
            await deleteSubcategory(subcategoryId)
            success(res, {}, 'Subcategoría eliminada con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router