import boom from '@hapi/boom'
import bcrypt from 'bcryptjs'
import { User } from '@/db/entities/User'
import { CreateUserOptions, UpdateUserOptions } from './userSchema'
import { QueryOptions } from '@/@types/common'
import { dataSource } from '@/bootstrap/dbBootstrap'

type UserQueryOptions = QueryOptions & { userStatus?: unknown }

export const getUsers = async (query: UserQueryOptions) => {
    const { skip, take, search, sortKey, sortOrder, userStatus } = query

    const queryBuilder = dataSource.createQueryBuilder(User, "user")
        .select([
            "user.id",
            "user.username",
            "user.fullname",
            "user.active",
            "user.role",
            "user.updatedAt"
        ])

    if (Array.isArray(userStatus) && userStatus.length > 0) {
        queryBuilder.andWhere("user.active IN (:...userStatus)", { userStatus })
    }

    if (search != undefined && search != '') {
        queryBuilder.andWhere(
            `(user.username ILIKE :search OR
            user.fullname ILIKE :search OR
            user.role ILIKE :search)`,
            { search: `%${search}%` }
        )
    }

    if (sortKey != undefined && (sortOrder === 'ASC' || sortOrder === 'DESC')) {
        queryBuilder.orderBy(sortKey, sortOrder)
    } else {
        queryBuilder.orderBy('user.updatedAt', 'DESC')
    }

    if (skip != undefined && take != undefined) {
        if (skip == 0) {
            queryBuilder.take(take)
        } else {
            queryBuilder.skip(skip).take(take)
        }
    }

    const [users, total] = await queryBuilder.getManyAndCount()

    return { users, total }
}

export const getUser = async (id: number) => {
    const user = await User.findOneBy({ id })

    if (!user) {
        throw boom.notFound('Usuario no existe')
    }

    return user
}

export const createUser = async (data: CreateUserOptions) => {
    const { username, password, fullname, role } = data
    const user = new User()
    const passwordHash = await bcrypt.hash(password, bcrypt.genSaltSync(10))

    user.username = username
    user.password = passwordHash
    user.fullname = fullname
    user.role = role

    await user.save()

    if (user) {
        delete user.password
        return user
    } else {
        throw boom.badData()
    }
}

export const updateUser = async (id: number, changes: UpdateUserOptions) => {
    const { username, password, fullname, role } = changes
    let passwordHash = undefined
    const user = await getUser(id)

    if(password) {
        passwordHash = await bcrypt.hash(password, bcrypt.genSaltSync(10))
    }

    user.username = username
    user.password = passwordHash
    user.fullname = fullname
    user.role = role

    await user.save()

    if (!user) {
        throw boom.badData('No se puede actualizar el usuario')
    }

    return user
}

export const getUserByUsername = async (username: string) => {
    const user = await User.findOneBy({ username })

    if(!user) {
        throw boom.unauthorized('Credenciales incorrectas')
    }
    
    return user
}

export const deleteUser = async (id: number) => {
    const user = await getUser(id)
    await user.remove()

    return user
}