import { Router, Request, Response, NextFunction } from 'express'
import { getUsers, getUser, createUser, deleteUser, updateUser } from './userController'
import { createUserScheme, updateUserScheme } from './userSchema'
import { queryCommonSchema, paramsCommonSchema } from '@/utils/commonSchema'
import { bodyValidator, queryValidator, paramsValidator } from '@/helpers/validatorHandler'
import { success } from '@/helpers/responseHandler'
import { queryParser } from '@/utils/queryParser'

const router = Router()

router.get('/',
    queryValidator(queryCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const query = queryParser(req.query)
            const users = await getUsers(query)
            success(res, users)
        } catch (error) {
            next(error)
        }
    }
)

router.get('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const userId = parseInt(req.params.id)
            const user = await getUser(userId)
            success(res, { user }, 'Usuario encontrado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

router.post('/',
    bodyValidator(createUserScheme),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            await createUser(req.body)
            success(res, {}, 'Usuario creado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.put('/:id',
    paramsValidator(paramsCommonSchema),
    bodyValidator(updateUserScheme),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const userId = Number(req.params.id)
            const user = await updateUser(userId, req.body)
            success(res, { user }, 'Usuario actualizado con éxito', 201)
        } catch (error) {
            next(error)
        }
    }
)

router.delete('/:id',
    paramsValidator(paramsCommonSchema),
    async (req: Request, res: Response, next: NextFunction) => {
        try {
            const userId = Number(req.params.id)
            await deleteUser(userId)
            success(res, {}, 'Usuario eliminado con éxito')
        } catch (error) {
            next(error)
        }
    }
)

export default router